# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: function_execution.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC, 6, 31, 1, '', 'function_execution.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x18\x66unction_execution.proto\x12\x0etargon.runtime\"\xf3\x01\n\x0e\x45xecuteRequest\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x0c\n\x04\x61rgs\x18\x02 \x01(\x0c\x12\x0e\n\x06kwargs\x18\x03 \x01(\x0c\x12\x10\n\x08input_id\x18\x04 \x01(\t\x12\x18\n\x10\x66unction_call_id\x18\x05 \x01(\t\x12\x12\n\ntimeout_ms\x18\x06 \x01(\x03\x12>\n\x08metadata\x18\x07 \x03(\x0b\x32,.targon.runtime.ExecuteRequest.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xec\x02\n\x0f\x45xecuteResponse\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x36\n\x06status\x18\x02 \x01(\x0e\x32&.targon.runtime.ExecuteResponse.Status\x12\x0e\n\x06result\x18\x03 \x01(\x0c\x12\r\n\x05\x65rror\x18\x04 \x01(\t\x12\x11\n\ttraceback\x18\x05 \x01(\t\x12\x19\n\x11\x65xecution_time_ms\x18\x06 \x01(\x03\x12?\n\x08metadata\x18\x07 \x03(\x0b\x32-.targon.runtime.ExecuteResponse.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"N\n\x06Status\x12\x0b\n\x07SUCCESS\x10\x00\x12\t\n\x05\x45RROR\x10\x01\x12\x0b\n\x07TIMEOUT\x10\x02\x12\r\n\tCANCELLED\x10\x03\x12\x10\n\x0cRATE_LIMITED\x10\x04\"%\n\x12HealthCheckRequest\x12\x0f\n\x07service\x18\x01 \x01(\t\"\xa9\x01\n\x13HealthCheckResponse\x12\x41\n\x06status\x18\x01 \x01(\x0e\x32\x31.targon.runtime.HealthCheckResponse.ServingStatus\"O\n\rServingStatus\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07SERVING\x10\x01\x12\x0f\n\x0bNOT_SERVING\x10\x02\x12\x13\n\x0fSERVICE_UNKNOWN\x10\x03\x32^\n\x10\x46unctionExecutor\x12J\n\x07\x45xecute\x12\x1e.targon.runtime.ExecuteRequest\x1a\x1f.targon.runtime.ExecuteResponse2\xae\x01\n\x06Health\x12P\n\x05\x43heck\x12\".targon.runtime.HealthCheckRequest\x1a#.targon.runtime.HealthCheckResponse\x12R\n\x05Watch\x12\".targon.runtime.HealthCheckRequest\x1a#.targon.runtime.HealthCheckResponse0\x01\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'function_execution_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals['_EXECUTEREQUEST_METADATAENTRY']._loaded_options = None
    _globals['_EXECUTEREQUEST_METADATAENTRY']._serialized_options = b'8\001'
    _globals['_EXECUTERESPONSE_METADATAENTRY']._loaded_options = None
    _globals['_EXECUTERESPONSE_METADATAENTRY']._serialized_options = b'8\001'
    _globals['_EXECUTEREQUEST']._serialized_start = 45
    _globals['_EXECUTEREQUEST']._serialized_end = 288
    _globals['_EXECUTEREQUEST_METADATAENTRY']._serialized_start = 241
    _globals['_EXECUTEREQUEST_METADATAENTRY']._serialized_end = 288
    _globals['_EXECUTERESPONSE']._serialized_start = 291
    _globals['_EXECUTERESPONSE']._serialized_end = 655
    _globals['_EXECUTERESPONSE_METADATAENTRY']._serialized_start = 241
    _globals['_EXECUTERESPONSE_METADATAENTRY']._serialized_end = 288
    _globals['_EXECUTERESPONSE_STATUS']._serialized_start = 577
    _globals['_EXECUTERESPONSE_STATUS']._serialized_end = 655
    _globals['_HEALTHCHECKREQUEST']._serialized_start = 657
    _globals['_HEALTHCHECKREQUEST']._serialized_end = 694
    _globals['_HEALTHCHECKRESPONSE']._serialized_start = 697
    _globals['_HEALTHCHECKRESPONSE']._serialized_end = 866
    _globals['_HEALTHCHECKRESPONSE_SERVINGSTATUS']._serialized_start = 787
    _globals['_HEALTHCHECKRESPONSE_SERVINGSTATUS']._serialized_end = 866
    _globals['_FUNCTIONEXECUTOR']._serialized_start = 868
    _globals['_FUNCTIONEXECUTOR']._serialized_end = 962
    _globals['_HEALTH']._serialized_start = 965
    _globals['_HEALTH']._serialized_end = 1139
# @@protoc_insertion_point(module_scope)
